--
-- PoettingerJumbo10000
-- Specialization for PoettingerJumbo10000 functionality
--
-- Modschmiede / LS-Modsource
-- @author  Manuel Leithner
-- @date  12/08/09
--

PoettingerJumbo10000 = {};

function PoettingerJumbo10000.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(TowBall, specializations);
end;

function PoettingerJumbo10000:load(xmlFile)

	self.setHydraulicDirection = SpecializationUtil.callSpecializationsFunction("setHydraulicDirection");

	local hydraulicsCount = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.hydraulics#count"), 0);	
	self.hydraulics = {};	
	for i=1, hydraulicsCount do
		local hydraulicName = string.format("vehicle.hydraulics.hydraulic%d", i);		
		self.hydraulics[i] = {};		
		self.hydraulics[i].node = Utils.indexToObject(self.components, getXMLString(xmlFile, hydraulicName .. "#index"));
		self.hydraulics[i].punch = Utils.indexToObject(self.components, getXMLString(xmlFile, hydraulicName .. "#punch"));
		self.hydraulics[i].translationPunch = Utils.indexToObject(self.components, getXMLString(xmlFile, hydraulicName .. "#punchFixpoint"));
		self.hydraulics[i].fixPoint = Utils.indexToObject(self.components, getXMLString(xmlFile, hydraulicName .. "#fixpoint"));	
		local ax, ay, az;		
		if self.hydraulics[i].punch ~= nil then
			ax, ay, az = getWorldTranslation(self.hydraulics[i].punch);
		else
			ax, ay, az = getWorldTranslation(self.hydraulics[i].node);
		end;
		if self.hydraulics[i].translationPunch ~= nil then
			local bx, by, bz = getWorldTranslation(self.hydraulics[i].translationPunch);		
			self.hydraulics[i].punchDistance = Utils.vector3Length(ax-bx, ay-by, az-bz);
		end;
	end;
	
	self.leftWheelSteeringIndex = 5;
	self.rightWheelSteeringIndex = 4;
	
	self.leftGuideRoller = {};
	self.leftGuideRoller.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.steeringTools#leftGuideRoller"));
	local ax, ay, az = getWorldTranslation(self.leftGuideRoller.node);
	local bx, by, bz = getWorldTranslation(self.hydraulics[5].node);
	local distance = Utils.vector3Length(ax-bx, ay-by, az-bz);
	self.leftGuideRoller.size = distance;
	local fixPoint = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.steeringTools#fixPoint"));
	local cx,cy,cz = getWorldTranslation(fixPoint);
	self.leftGuideRoller.distance = Utils.vector3Length(cx-bx, cy-by, cz-bz);
	self.rightGuideRoller = {};
	self.rightGuideRoller.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.steeringTools#rightGuideRoller"));
	ax, ay, az = getWorldTranslation(self.rightGuideRoller.node);
	bx, by, bz = getWorldTranslation(self.hydraulics[4].node);
	self.leftGuideRoller.size = Utils.vector3Length(ax-bx, ay-by, az-bz);
	
	
	
	
	self.hayBase = {};
	self.hayBase.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.hayBase#index"));
	self.hayBase.unloadingSpeed = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.hayBase#unloadingSpeed"), 0.001);
	self.hayBase.loadingSpeed = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.hayBase#loadingSpeed"), 0.0005);
	self.hayBase.speed = self.hayBase.loadingSpeed;
	self.hayBase.unloadingDirection = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.hayBase#unloadingDirection"), 1);
	self.hayBase.offset = 0;
	setShaderParameter(self.hayBase.node, "movingDirection", 0,0,0,0,false);
	
	self.pickup = {};
	self.pickup.minRot = {Utils.degToRad(getXMLFloat(xmlFile, "vehicle.pickup#minRotLimit"))};
	self.pickup.maxRot = {Utils.degToRad(getXMLFloat(xmlFile, "vehicle.pickup#maxRotLimit"))};
	self.pickup.rotTime = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.pickup#rotTime"),2) * 1000;
	self.pickup.rake = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.pickup#rake"));
	self.pickup.drum = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.pickup#drum"));
	self.pickup.currentRot = {0};
	self.pickup.movingDirection = 0;
	
	self.drawBar = {};
	self.drawBar.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.drawBar#index"));
	local x = Utils.degToRad(getXMLFloat(xmlFile, "vehicle.drawBar#minRot"));
	self.drawBar.minRot = {x, 0, 0};
	x = Utils.degToRad(getXMLFloat(xmlFile, "vehicle.drawBar#maxRot"));
	self.drawBar.maxRot = {x, 0, 0};
	self.drawBar.rotTime = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.drawBar#rotTime"), 2) * 1000;
	self.drawBar.isDown = true;
	
	self.setHydraulicTime = 30;
	self.doTractorFixPointSearch = true;
	
	
    local forageWgnSound = getXMLString(xmlFile, "vehicle.forageWgnSound#file");
    if forageWgnSound ~= nil and forageWgnSound ~= "" then
        forageWgnSound = Utils.getFilename(forageWgnSound, self.baseDirectory);
        self.forageWgnSound = createSample("forageWgnSound");
        self.forageWgnSoundEnabled = false;
        loadSample(self.forageWgnSound, forageWgnSound, false);
        self.forageWgnSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.forageWgnSound#pitchOffset"), 1);
        self.forageWgnSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.forageWgnSound#volume"), 1.0);
    end;
	
    self.fillScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fillScale#value"), 1);
    self.wasToFast = false;
    self.isTurnedOn = false;
	self.isHayBaseActive = false;
end;

function PoettingerJumbo10000:delete()
    if self.forageWgnSound ~= nil then
        delete(self.forageWgnSound);
    end;
end;

function PoettingerJumbo10000:mouseEvent(posX, posY, isDown, isUp, button)
end;

function PoettingerJumbo10000:keyEvent(unicode, sym, modifier, isDown)
end;

function PoettingerJumbo10000:update(dt)	

	if self:getIsActive() then
		local x1,y1,z1 = localDirectionToWorld(self.attacherVehicle.rootNode, 0, 0, 1)
		local x2,y2,z2 = localDirectionToWorld(self.rootNode, 0, 0, 1)
		local dot = x1*x2 + z1*z2;
		dot = dot / (Utils.vector2Length(x1,z1) * Utils.vector2Length(x2,z2))
		angle = math.acos(dot);
		local cy = z1 * x2 - x1 * z2;		
		if cy < 0 then
			angle = -angle;
		end;
		backAngle  = 24 * (angle / 50);
		setRotation(self.leftGuideRoller.node, -backAngle, 0,0);
		setRotation(self.rightGuideRoller.node, backAngle, 0,0);
	end;

	self.pickup.movingDirection = 0;
	
	if self:getIsActiveForInput() then
		if self.vehicleJoint ~= nil then
			if InputBinding.isPressed(InputBinding.LOWER_JUMBO) then
				self.pickup.movingDirection = 1;
			end;
			if InputBinding.isPressed(InputBinding.LIFT_JUMBO) then
				self.pickup.movingDirection = -1;
			end;			
		end;
		if InputBinding.hasEvent(InputBinding.LOWER_IMPLEMENT) then
			self.pickup.isDown = not self.pickup.isDown;
		end;
        if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA) then
            self.isTurnedOn = not self.isTurnedOn;
			if not self.isTurnedOn then
				self.isHayBaseActive = false;
			end;
        end;
    end;
	
	if self.isHayBaseActive then
		self.hayBase.offset = self.hayBase.offset + self.hayBase.speed*dt;
		setShaderParameter(self.hayBase.node, "movingDirection", self.hayBase.unloadingDirection*self.hayBase.offset,0,0,0,false);
	end;	
	
	self.wasToFast = false;
	local pickupIsDown = false;
	
	if self:getIsActive() then
	
		if self.isTurnedOn then
			rotate(self.pickup.rake, -0.6 * dt, 0, 0);
		end;
		
		if self.pickup.currentRot[1] > (self.pickup.maxRot[1] / 2) then
			pickupIsDown = true;
		end;	
	
		self.setHydraulicTime = 30;
		
		if self.leftFixPoint ~= nil then
			local x,y,z = getRotation(self.hydraulics[self.leftWheelSteeringIndex].node);
			--print(math.deg(x));
			x = 90 + math.deg(x);

			local ax,ay,az = getWorldTranslation(self.leftFixPoint);
			local bx,by,bz = getWorldTranslation(self.hydraulics[self.leftWheelSteeringIndex].node);
			local distance = Utils.vector3Length(ax-bx, ay-by, az-bz) - self.leftGuideRoller.distance;
			local c = math.sqrt(distance*distance + self.leftGuideRoller.size*self.leftGuideRoller.size - 2*distance*self.leftGuideRoller.size*math.cos(x));
			local angle = math.acos((self.leftGuideRoller.size*self.leftGuideRoller.size + c*c - distance*distance) / 2*self.leftGuideRoller.size+c);
		end;
		
		if self.isTurnedOn  then
            local toFast = false;
			if pickupIsDown then
				toFast = self:doCheckSpeedLimit() and self.attacherVehicle.lastSpeed*3600 > 29;		

				if not toFast and self.capacity > self.fillLevel then
					local totalArea =0;
					local usedFruitType = FruitUtil.FRUITTYPE_UNKNOWN;
					for k, cuttingArea in pairs(self.cuttingAreas) do
						local x,y,z = getWorldTranslation(cuttingArea.start);
						local x1,y1,z1 = getWorldTranslation(cuttingArea.width);
						local x2,y2,z2 = getWorldTranslation(cuttingArea.height);

						for fruitType,v in pairs(self.fillTypes) do
							if fruitType ~= FruitUtil.FRUITTYPE_UNKNOWN then
								if self:allowFillType(fruitType) then
									local area = Utils.updateFruitWindrowArea(fruitType, x, z, x1, z1, x2, z2, 0)*g_currentMission.windrowCutLongRatio;
									area = area + Utils.updateFruitCutLongArea(fruitType, x, z, x1, z1, x2, z2, 0);
									if area > 0 then
										totalArea = totalArea+area;
										usedFruitType = fruitType;
									end;
								end;
							end;
						end;
					end;
					
					if totalArea > 0 then
						local pixelToQm = 2048 / 4096 * 2048 / 4096  / g_currentMission.maxFruitValue; -- 4096px are mapped to 2048m
						local literPerQm = FruitUtil.fruitIndexToDesc[usedFruitType].literPerQm * (1 + 0.5 * (3 - g_currentMission.missionStats.difficulty));
	                    local qm = totalArea*pixelToQm;
	                    local deltaLevel = qm*literPerQm * self.fillScale;
	                    self:setFillLevel(self.fillLevel+deltaLevel, usedFruitType);
					end;
				end;			
			end;

            self.wasToFast = toFast;
		end;

		if self.isTurnedOn or self.isHayBaseActive then
			if not self.forageWgnSoundEnabled and self:getIsActiveForSound() then
				playSample(self.forageWgnSound, 0, self.forageWgnSoundVolume, 0);
				setSamplePitch(self.forageWgnSound, self.forageWgnSoundPitchOffset);
				self.forageWgnSoundEnabled = true;
			end;
		end;
		
		
        if self.forageWgnSoundEnabled and (not self.isTurnedOn and not self.isHayBaseActive) then
            stopSample(self.forageWgnSound);
            self.forageWgnSoundEnabled = false;
        end;		
		
		local newRotLimit = Utils.getMovedLimitedValues(self.pickup.currentRot, self.pickup.maxRot, self.pickup.minRot, 1, self.pickup.rotTime, dt, not self.pickup.isDown);
		if math.abs(newRotLimit[1] - self.pickup.currentRot[1]) > 0.001 then	
			setJointRotationLimit(self.componentJoints[1].jointIndex, 0, true, 0, newRotLimit[1]);		
		end;		
		self.pickup.currentRot = newRotLimit;		
		
		if self.doTractorFixPointSearch  and self.leftFixPoint ~= nil and self.rightFixPoint ~= nil then
			self.hydraulics[self.leftWheelSteeringIndex].fixPoint = self.leftFixPoint;
			self.hydraulics[self.rightWheelSteeringIndex].fixPoint = self.rightFixPoint;
			self.doTractorFixPointSearch  = false;
		end;
		
		if self.vehicleJoint ~= nil and self.pickup.movingDirection ~= 0 then
			local x, y, z = getRotation(self.drawBar.node);	
			local rot = {x,y,z};
			local newRot = Utils.getMovedLimitedValues(rot, self.drawBar.maxRot, self.drawBar.minRot, 3, self.drawBar.rotTime, dt, self.pickup.movingDirection == 1);
			setRotation(self.drawBar.node, unpack(newRot));		
			
			if math.abs(x-newRot[1]) > 0.001 then
				setJointFrame(self.vehicleJoint.jointIndex, 1, self.attacherJoint.node);
				self:setHydraulicDirection(1);
				if not self.hydraulicSoundEnabled and self.hydraulicSound ~= nil and self:getIsActiveForSound() then
		            playSample(self.hydraulicSound, 0, self.hydraulicSoundVolume, 0);
		            setSamplePitch(self.hydraulicSound, self.hydraulicSoundPitchOffset-0.4);
		            self.hydraulicSoundEnabled = true;   
		        end;
			end;
		else
			if self.tipState ~= RealLoading.TIPSTATE_OPENING and self.tipState ~= RealLoading.TIPSTATE_CLOSING then
				if self.hydraulicSoundEnabled then
					stopSample(self.hydraulicSound);
					self.hydraulicSoundEnabled = false;   
				end;
			end;
		end;
	end;
	
	if self.setHydraulicTime > 0 then
		for i=2, 5 do 
			self:setHydraulicDirection(i);
		end;
		self.setHydraulicTime = self.setHydraulicTime - 1;
	end;
end;

function PoettingerJumbo10000:draw()
    if self.wasToFast then
        g_currentMission:addWarning(g_i18n:getText("Dont_drive_to_fast") .. "\n" .. string.format(g_i18n:getText("Cruise_control_levelN"), "2", InputBinding.getButtonKeyName(InputBinding.SPEED_LEVEL2)), 0.07+0.022, 0.019+0.029);
    end;
    if self.isTurnedOn then
        g_currentMission:addHelpButtonText(string.format(g_i18n:getText("turn_off_OBJECT"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA);
    else
        g_currentMission:addHelpButtonText(string.format(g_i18n:getText("turn_on_OBJECT"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA);
    end;
    g_currentMission:addExtraPrintText(string.format(g_i18n:getText("lower_lift_jumbo"), self.typeDesc) .. g_i18n:getText("Key") .. " " .. InputBinding.getButtonKeyName(InputBinding.LOWER_JUMBO) .. "/" .. InputBinding.getButtonKeyName(InputBinding.LIFT_JUMBO));
    if self.pickup.isDown then
        g_currentMission:addHelpButtonText(string.format(g_i18n:getText("lift_pickup_jumbo"), self.typeDesc), InputBinding.LOWER_IMPLEMENT);
    else
        g_currentMission:addHelpButtonText(string.format(g_i18n:getText("lower_pickup_jumbo"), self.typeDesc), InputBinding.LOWER_IMPLEMENT);
    end;	
end;

function PoettingerJumbo10000:setHydraulicDirection(index)
	local hydraulic = self.hydraulics[index];

	if hydraulic.fixPoint ~= nil then
		local ax, ay, az = getWorldTranslation(hydraulic.node);
		local bx, by, bz = getWorldTranslation(hydraulic.fixPoint);
		local x, y, z = worldDirectionToLocal(getParent(hydraulic.node), bx-ax, by-ay, bz-az);
		
		setDirection(hydraulic.node, x, y, z, 0, 1, 0);
		if hydraulic.punch ~= nil then
			local distance = Utils.vector3Length(ax-bx, ay-by, az-bz);
			setTranslation(hydraulic.punch, 0, 0, distance-hydraulic.punchDistance);
		end;
	end;
end;


function PoettingerJumbo10000:onAttach(attacherVehicle)
	self.doJointSearch = true;
end;

function PoettingerJumbo10000:onDetach()
	self.vehicleJoint = nil;
	self.doTractorFixPointSearch = true;
	self.hydraulics[self.leftWheelSteeringIndex].fixPoint = nil;
	self.hydraulics[self.rightWheelSteeringIndex].fixPoint = nil;
	
	if self.deactivateOnDetach then
        PoettingerJumbo10000.onDeactivate(self);
    else
        PoettingerJumbo10000.onDeactivateSounds(self);
    end;
end;

function PoettingerJumbo10000:onLeave()
    if self.deactivateOnLeave then
        PoettingerJumbo10000.onDeactivate(self);
    else
        PoettingerJumbo10000.onDeactivateSounds(self);
    end;
end;

function PoettingerJumbo10000:onDeactivate()
    self.isTurnedOn = false;
	self.isHayBaseActive = false;
    PoettingerJumbo10000.onDeactivateSounds(self);
end;

function PoettingerJumbo10000:onDeactivateSounds()
    if self.forageWgnSoundEnabled then
        stopSample(self.forageWgnSound);
        self.forageWgnSoundEnabled = false;
    end;
end;